#ifndef MPI_UTILS_H__
#define MPI_UTILS_H__

#include <stdio.h>  /* fprintf */
#include <stdlib.h> /* exit, EXIT_* */
#include <mpi.h>

/* POSIX SPECIFIC DEBUG */
#ifdef DEBUG_BUILD
# include <sys/types.h> /* pid_t */
# include <unistd.h>    /* getpid, hostname */

# define DEBUG_PID_ATTACH			\
  do {						\
    volatile int i = 0;				\
    char hostname[128];				\
						\
    gethostname(hostname, sizeof(hostname));	\
						\
    printf("PID %d on %s ready for attach.\n",	\
	   getpid(), hostname);			\
    fflush(stdout);				\
						\
    while (!i)					\
      sleep(1);					\
  } while (0)
#else /* NOT DEBUG_BUILD */
# define DEBUG_PID_ATTACH ((void) 0)
#endif /* DEBUG_BUILD */
/* MPI */

#define ROOT_PROCESS 0

#define PROCESS_IS_ROOT(id) \
  ((id) == ROOT_PROCESS)

/* MPI-UTILS */

#define VALID_PROCESS(id, p) \
  ( ((id) < 0) || ((id) >= (p)) ? MPI_PROC_NULL : (id) )

#define BLOCK_LOW(id, p, n) \
  ((id)*(n)/(p))

#define BLOCK_HIGH(id, p, n) \
  (BLOCK_LOW((id)+1, (p), (n)))

#define BLOCK_SIZE(id, p, n) \
  ((BLOCK_HIGH((id), (p), (n))) - (BLOCK_LOW((id), (p), (n))))

#define BLOCK_OWNER(i, p, n) \
  (((p)*((i)+1)-1)/(n))

#define FINALIZE_AND_EXIT(status) \
  do { MPI_Finalize(); exit(status); } while (0)

#define FINALIZE_AND_FAIL \
  FINALIZE_AND_EXIT(EXIT_FAILURE)

#define FINALIZE_AND_SUCCEED \
  FINALIZE_AND_EXIT(EXIT_SUCCESS)

#define root_fprintf(id, ...) \
  (PROCESS_IS_ROOT(id) ? fprintf(__VA_ARGS__) : 0)

#define root_printf(id, ...) \
  root_fprintf((id), stdout, __VA_ARGS__)

#define root_eprintf(id, ...) \
  root_fprintf((id), stderr, __VA_ARGS__)

#endif /* MPI_UTILS_H__ */
